@@ -63,13 +63,13 @@ int lookup_init(const char *mapfmt, int argc, const char *const *argv, void **co
 	ctxt = malloc(sizeof(struct lookup_context));
 	if (!ctxt) {
 		char *estr = strerror_r(errno, buf, MAX_ERR_BUF);
-		crit(LOGOPT_ANY, MODPREFIX "malloc: %s", estr);
+		logerr(MODPREFIX "malloc: %s", estr);
 		return 1;
 	}
 
 	if (argc < 1) {
 		free(ctxt);
-		crit(LOGOPT_ANY, MODPREFIX "No map name");
+		logerr(MODPREFIX "No map name");
 		return 1;
 	}
 
@@ -77,21 +77,21 @@ int lookup_init(const char *mapfmt, int argc, const char *const *argv, void **co
 
 	if (ctxt->mapname[0] != '/') {
 		free(ctxt);
-		msg(MODPREFIX "file map %s is not an absolute pathname",
-		    argv[0]);
+		logmsg(MODPREFIX
+		     "file map %s is not an absolute pathname", argv[0]);
 		return 1;
 	}
 
 	if (access(ctxt->mapname, R_OK)) {
 		free(ctxt);
-		msg(MODPREFIX "file map %s missing or not readable",
-		    argv[0]);
+		warn(LOGOPT_NONE, MODPREFIX
+		    "file map %s missing or not readable", argv[0]);
 		return 1;
 	}
 
 	if (stat(ctxt->mapname, &st)) {
 		free(ctxt);
-		crit(LOGOPT_ANY, MODPREFIX "file map %s, could not stat",
+		logmsg(MODPREFIX "file map %s, could not stat",
 		     argv[0]);
 		return 1;
 	}
@@ -104,7 +104,7 @@ int lookup_init(const char *mapfmt, int argc, const char *const *argv, void **co
 	ctxt->parse = open_parse(mapfmt, MODPREFIX, argc - 1, argv + 1);
 	if (!ctxt->parse) {
 		free(ctxt);
-		crit(LOGOPT_ANY, MODPREFIX "failed to open parse context");
+		logmsg(MODPREFIX "failed to open parse context");
 		return 1;
 	}
 	*context = ctxt;
@@ -112,7 +112,7 @@ int lookup_init(const char *mapfmt, int argc, const char *const *argv, void **co
 	return 0;
 }
 
-static int read_one(FILE *f, char *key, unsigned int *k_len, char *mapent, unsigned int *m_len)
+static int read_one(unsigned logopt, FILE *f, char *key, unsigned int *k_len, char *mapent, unsigned int *m_len)
 {
 	char *kptr, *p;
 	int mapent_len, key_len;
@@ -193,7 +193,7 @@ static int read_one(FILE *f, char *key, unsigned int *k_len, char *mapent, unsig
 				if (gotten == got_plus)
 					goto got_it;
 				else if (escape == esc_all) {
-					warn(LOGOPT_ANY, MODPREFIX
+					warn(logopt, MODPREFIX
 					    "unmatched \" in map key %s", key);
 					goto next;
 				} else if (escape != esc_val)
@@ -208,7 +208,7 @@ static int read_one(FILE *f, char *key, unsigned int *k_len, char *mapent, unsig
 				if (key_len == KEY_MAX_LEN) {
 					state = st_badent;
 					gotten = got_nothing;
-					warn(LOGOPT_ANY,
+					warn(logopt,
 					      MODPREFIX "map key \"%s...\" "
 					      "is too long.  The maximum key "
 					      "length is %d", key,
@@ -245,7 +245,7 @@ static int read_one(FILE *f, char *key, unsigned int *k_len, char *mapent, unsig
 				state = st_begin;
 				if (gotten == got_real || gotten == getting)
 					goto got_it;
-				warn(LOGOPT_ANY, MODPREFIX 
+				warn(logopt, MODPREFIX 
 				      "bad map entry \"%s...\" for key "
 				      "\"%s\"", mapent, key);
 				goto next;
@@ -286,7 +286,7 @@ static int read_one(FILE *f, char *key, unsigned int *k_len, char *mapent, unsig
 			if (ch == '\n') {
 				if (escape == esc_all) {
 					state = st_begin;
-					warn(LOGOPT_ANY, MODPREFIX
+					warn(logopt, MODPREFIX
 					     "unmatched \" in %s for key %s",
 					     mapent, key);
 					goto next;
@@ -310,7 +310,7 @@ static int read_one(FILE *f, char *key, unsigned int *k_len, char *mapent, unsig
 				   	goto got_it;
 				ungetc(nch, f);
 			} else {
-				warn(LOGOPT_ANY,
+				warn(logopt,
 				      MODPREFIX "map entry \"%s...\" for key "
 				      "\"%s\" is too long.  The maximum entry"
 				      " size is %d", mapent, key,
@@ -388,6 +388,7 @@ int lookup_read_master(struct master *master, time_t age, void *context)
 	struct lookup_context *ctxt = (struct lookup_context *) context;
 	unsigned int timeout = master->default_timeout;
 	unsigned int logging = master->default_logging;
+	unsigned int logopt = master->logopt;
 	char *buffer;
 	int blen;
 	char *path;
@@ -402,29 +403,28 @@ int lookup_read_master(struct master *master, time_t age, void *context)
 		return NSS_STATUS_UNAVAIL;
 
 	if (master->depth > MAX_INCLUDE_DEPTH) {
-		error(LOGOPT_ANY,
-		      MODPREFIX
+		error(logopt, MODPREFIX
 		      "maximum include depth exceeded %s", master->name);
 		return NSS_STATUS_UNAVAIL;
 	}
 
 	path = alloca(KEY_MAX_LEN + 1);
 	if (!path) {
-		error(LOGOPT_ANY,
+		error(logopt,
 		      MODPREFIX "could not malloc storage for path");
 		return NSS_STATUS_UNAVAIL;
 	}
 
 	ent = alloca(MAPENT_MAX_LEN + 1);
 	if (!ent) {
-		error(LOGOPT_ANY,
+		error(logopt,
 		      MODPREFIX "could not malloc storage for mapent");
 		return NSS_STATUS_UNAVAIL;
 	}
 
 	f = fopen(ctxt->mapname, "r");
 	if (!f) {
-		error(LOGOPT_ANY,
+		error(logopt,
 		      MODPREFIX "could not open master map file %s",
 		      ctxt->mapname);
 		return NSS_STATUS_UNAVAIL;
@@ -438,14 +438,19 @@ int lookup_read_master(struct master *master, time_t age, void *context)
 	}
 
 	while(1) {
-		entry = read_one(f, path, &path_len, ent, &ent_len);
+		entry = read_one(logopt, f, path, &path_len, ent, &ent_len);
 		if (!entry) {
 			if (feof(f))
 				break;
+			if (ferror(f)) {
+				warn(logopt, MODPREFIX
+				     "error reading map %s", ctxt->mapname);
+				break;
+			}
 			continue;
 		}
 
-		debug(LOGOPT_NONE, MODPREFIX "read entry %s", path);
+		debug(logopt, MODPREFIX "read entry %s", path);
 
 		/*
 		 * If key starts with '+' it has to be an
@@ -464,21 +469,29 @@ int lookup_read_master(struct master *master, time_t age, void *context)
 				master->recurse = 1;;
 			master->depth++;
 			status = lookup_nss_read_master(master, age);
-			if (!status)
-				warn(LOGOPT_ANY,
+			if (!status) {
+				warn(logopt,
 				     MODPREFIX
 				     "failed to read included master map %s",
 				     master->name);
+				if (!master->recurse) {
+					master->depth--;
+					master->recurse = 0;
+					fclose(f);
+					return NSS_STATUS_UNAVAIL;
+				}
+			}
 			master->depth--;
 			master->recurse = 0;
 
 			master->name = save_name;
 		} else {
-			blen = path_len + 1 + ent_len + 1;
+			blen = path_len + 1 + ent_len + 2;
 			buffer = malloc(blen);
 			if (!buffer) {
-				error(LOGOPT_ANY,
+				error(logopt,
 				      MODPREFIX "could not malloc parse buffer");
+				fclose(f);
 				return NSS_STATUS_UNAVAIL;
 			}
 			memset(buffer, 0, blen);
@@ -498,7 +511,7 @@ int lookup_read_master(struct master *master, time_t age, void *context)
 	}
 
 	if (fstat(fd, &st)) {
-		crit(LOGOPT_ANY, MODPREFIX "file map %s, could not stat",
+		crit(logopt, MODPREFIX "file map %s, could not stat",
 		       ctxt->mapname);
 		return NSS_STATUS_UNAVAIL;
 	}
@@ -604,7 +617,13 @@ prepare_plus_include(struct autofs_point *ap, time_t age, char *key, unsigned in
 	argv[1] = NULL;
 
 	source = master_find_source_instance(current, type, fmt, argc, argv);
-	if (!source) {
+	if (source)
+		/*
+		 * Make sure included map age is in sync with its owner
+		 * or we could incorrectly wipe out its entries.
+		 */
+		source->age = age;
+	else {
 		source = master_add_source_instance(current, type, fmt, age, argc, argv);
 		if (!source) {
 			free(buf);
@@ -679,10 +698,15 @@ int lookup_read_map(struct autofs_point *ap, time_t age, void *context)
 	}
 
 	while(1) {
-		entry = read_one(f, key, &k_len, mapent, &m_len);
+		entry = read_one(ap->logopt, f, key, &k_len, mapent, &m_len);
 		if (!entry) {
 			if (feof(f))
 				break;
+			if (ferror(f)) {
+				warn(ap->logopt, MODPREFIX
+				      "error reading map %s", ctxt->mapname);
+				break;
+			}
 			continue;
 		}
 			
@@ -711,9 +735,12 @@ int lookup_read_map(struct autofs_point *ap, time_t age, void *context)
 
 			/* Gim'ee some o' that 16k stack baby !! */
 			status = lookup_nss_read_map(ap, inc_source, age);
-			if (!status)
+			if (!status) {
 				warn(ap->logopt,
 				     "failed to read included map %s", key);
+				fclose(f);
+				return NSS_STATUS_UNAVAIL;
+			}
 		} else {
 			char *s_key; 
 
@@ -781,7 +808,7 @@ static int lookup_one(struct autofs_point *ap,
 	}
 
 	while(1) {
-		entry = read_one(f, mkey, &k_len, mapent, &m_len);
+		entry = read_one(ap->logopt, f, mkey, &k_len, mapent, &m_len);
 		if (entry) {
 			/*
 			 * If key starts with '+' it has to be an
@@ -848,6 +875,12 @@ static int lookup_one(struct autofs_point *ap,
 
 		if (feof(f))
 			break;
+
+		if (ferror(f)) {
+			warn(ap->logopt, MODPREFIX
+			      "error reading map %s", ctxt->mapname);
+			break;
+		}		
 	}
 
 	fclose(f);
@@ -888,7 +921,7 @@ static int lookup_wild(struct autofs_point *ap, struct lookup_context *ctxt)
 	}
 
 	while(1) {
-		entry = read_one(f, mkey, &k_len, mapent, &m_len);
+		entry = read_one(ap->logopt, f, mkey, &k_len, mapent, &m_len);
 		if (entry) {
 			int eq;
 
@@ -907,6 +940,12 @@ static int lookup_wild(struct autofs_point *ap, struct lookup_context *ctxt)
 
 		if (feof(f))
 			break;
+
+		if (ferror(f)) {
+			warn(ap->logopt, MODPREFIX
+			      "error reading map %s", ctxt->mapname);
+			break;
+		}		
 	}
 
 	fclose(f);
@@ -1025,7 +1064,7 @@ int lookup_mount(struct autofs_point *ap, const char *name, int name_len, void *
 
 	debug(ap->logopt, MODPREFIX "looking up %s", name);
 
-	key_len = snprintf(key, KEY_MAX_LEN, "%s", name);
+	key_len = snprintf(key, KEY_MAX_LEN + 1, "%s", name);
 	if (key_len > KEY_MAX_LEN)
 		return NSS_STATUS_NOTFOUND;
 
@@ -1066,8 +1105,9 @@ int lookup_mount(struct autofs_point *ap, const char *name, int name_len, void *
 			if (status == NSS_STATUS_COMPLETED)
 				return NSS_STATUS_SUCCESS;
 
-			debug(ap->logopt,
-			      MODPREFIX "check indirect map lookup failed");
+			error(ap->logopt,
+			      MODPREFIX "key \"%s\" not found in map",
+			      name);
 
 			return NSS_STATUS_NOTFOUND;
 		}
@@ -1104,11 +1144,13 @@ int lookup_mount(struct autofs_point *ap, const char *name, int name_len, void *
 				rv = cache_update(mc, source, key, NULL, now);
 			if (rv != CHE_FAIL) {
 				me = cache_lookup_distinct(mc, key);
-				me->status = now + NEGATIVE_TIMEOUT;
+				me->status = now + ap->negative_timeout;
 			}
 			cache_unlock(mc);
 		}
-	}
+	} else
+		error(ap->logopt,
+		      MODPREFIX "key \"%s\" not found in map.", name);
 
 	if (ret)
 		return NSS_STATUS_TRYAGAIN;
